@@ -3,6 +3,7 @@ from decimal import Decimal
 import pytest
 
 from pydantic_core import PydanticCustomError, PydanticKindError, PydanticOmit, SchemaValidator, ValidationError
+from pydantic_core._pydantic_core import list_all_errors
 
 from .conftest import PyAndJson
 
@@ -165,100 +166,105 @@ def test_pydantic_error_kind_raise_ctx():
     ]
 
 
-@pytest.mark.parametrize(
-    'kind, message, context',
-    [
-        ('json_invalid', 'Invalid JSON: foobar', {'error': 'foobar'}),
-        ('recursion_loop', 'Recursion error - cyclic reference detected', None),
-        ('dict_attributes_type', 'Input should be a valid dictionary or instance to extract fields from', None),
-        ('missing', 'Field required', None),
-        ('frozen', 'Field is frozen', None),
-        ('extra_forbidden', 'Extra inputs are not permitted', None),
-        ('invalid_key', 'Keys should be strings', None),
-        ('get_attribute_error', 'Error extracting attribute: foo', {'error': 'foo'}),
-        ('model_class_type', 'Input should be an instance of foo', {'class_name': 'foo'}),
-        ('none_required', 'Input should be None/null', None),
-        ('bool', 'Input should be a valid boolean', None),
-        ('greater_than', 'Input should be greater than 42.1', {'gt': 42.1}),
-        ('greater_than', 'Input should be greater than 42.1', {'gt': '42.1'}),
-        ('greater_than', 'Input should be greater than 2020-01-01', {'gt': '2020-01-01'}),
-        ('greater_than_equal', 'Input should be greater than or equal to 42.1', {'ge': 42.1}),
-        ('less_than', 'Input should be less than 42.1', {'lt': 42.1}),
-        ('less_than_equal', 'Input should be less than or equal to 42.1', {'le': 42.1}),
-        ('finite_number', 'Input should be a finite number', None),
-        (
-            'too_short',
-            'Foobar should have at least 42 items after validation, not 40',
-            {'field_type': 'Foobar', 'min_length': 42, 'actual_length': 40},
-        ),
-        (
-            'too_long',
-            'Foobar should have at most 42 items after validation, not 50',
-            {'field_type': 'Foobar', 'max_length': 42, 'actual_length': 50},
-        ),
-        ('string_type', 'Input should be a valid string', None),
-        ('string_unicode', 'Input should be a valid string, unable to parse raw data as a unicode string', None),
-        ('string_pattern_mismatch', "String should match pattern 'foo'", {'pattern': 'foo'}),
-        ('string_too_short', 'String should have at least 42 characters', {'min_length': 42}),
-        ('string_too_long', 'String should have at most 42 characters', {'max_length': 42}),
-        ('dict_type', 'Input should be a valid dictionary', None),
-        ('dict_from_mapping', 'Unable to convert mapping to a dictionary, error: foobar', {'error': 'foobar'}),
-        ('iteration_error', 'Error iterating over object, error: foobar', {'error': 'foobar'}),
-        ('list_type', 'Input should be a valid list/array', None),
-        ('tuple_type', 'Input should be a valid tuple', None),
-        ('set_type', 'Input should be a valid set', None),
-        ('bool_type', 'Input should be a valid boolean', None),
-        ('bool_parsing', 'Input should be a valid boolean, unable to interpret input', None),
-        ('int_type', 'Input should be a valid integer', None),
-        ('int_parsing', 'Input should be a valid integer, unable to parse string as an integer', None),
-        ('int_from_float', 'Input should be a valid integer, got a number with a fractional part', None),
-        ('multiple_of', 'Input should be a multiple of 42.1', {'multiple_of': 42.1}),
-        ('greater_than', 'Input should be greater than 42.1', {'gt': 42.1}),
-        ('greater_than_equal', 'Input should be greater than or equal to 42.1', {'ge': 42.1}),
-        ('less_than', 'Input should be less than 42.1', {'lt': 42.1}),
-        ('less_than_equal', 'Input should be less than or equal to 42.1', {'le': 42.1}),
-        ('float_type', 'Input should be a valid number', None),
-        ('float_parsing', 'Input should be a valid number, unable to parse string as an number', None),
-        ('bytes_type', 'Input should be a valid bytes', None),
-        ('bytes_too_short', 'Data should have at least 42 bytes', {'min_length': 42}),
-        ('bytes_too_long', 'Data should have at most 42 bytes', {'max_length': 42}),
-        ('value_error', 'Value error, foobar', {'error': 'foobar'}),
-        ('assertion_error', 'Assertion failed, foobar', {'error': 'foobar'}),
-        ('literal_error', 'Input should be foo', {'expected': 'foo'}),
-        ('literal_error', 'Input should be foo or bar', {'expected': 'foo or bar'}),
-        ('date_type', 'Input should be a valid date', None),
-        ('date_parsing', 'Input should be a valid date in the format YYYY-MM-DD, foobar', {'error': 'foobar'}),
-        ('date_from_datetime_parsing', 'Input should be a valid date or datetime, foobar', {'error': 'foobar'}),
-        ('date_from_datetime_inexact', 'Datetimes provided to dates should have zero time - e.g. be exact dates', None),
-        ('time_type', 'Input should be a valid time', None),
-        ('time_parsing', 'Input should be in a valid time format, foobar', {'error': 'foobar'}),
-        ('datetime_type', 'Input should be a valid datetime', None),
-        ('datetime_parsing', 'Input should be a valid datetime, foobar', {'error': 'foobar'}),
-        ('datetime_object_invalid', 'Invalid datetime object, got foobar', {'error': 'foobar'}),
-        ('time_delta_type', 'Input should be a valid timedelta', None),
-        ('time_delta_parsing', 'Input should be a valid timedelta, foobar', {'error': 'foobar'}),
-        ('frozen_set_type', 'Input should be a valid frozenset', None),
-        ('is_instance_of', 'Input should be an instance of Foo', {'class': 'Foo'}),
-        ('is_subclass_of', 'Input should be a subclass of Foo', {'class': 'Foo'}),
-        ('callable_type', 'Input should be callable', None),
-        (
-            'union_tag_invalid',
-            "Input tag 'foo' found using bar does not match any of the expected tags: baz",
-            {'discriminator': 'bar', 'tag': 'foo', 'expected_tags': 'baz'},
-        ),
-        ('union_tag_not_found', 'Unable to extract tag using discriminator foo', {'discriminator': 'foo'}),
-        (
-            'arguments_type',
-            'Arguments must be a tuple of (positional arguments, keyword arguments) or a plain dict',
-            None,
-        ),
-        ('unexpected_keyword_argument', 'Unexpected keyword argument', None),
-        ('missing_keyword_argument', 'Missing required keyword argument', None),
-        ('unexpected_positional_argument', 'Unexpected positional argument', None),
-        ('missing_positional_argument', 'Missing required positional argument', None),
-        ('multiple_argument_values', 'Got multiple values for argument', None),
-    ],
-)
+all_errors = [
+    ('json_invalid', 'Invalid JSON: foobar', {'error': 'foobar'}),
+    ('json_type', 'JSON input should be str, bytes or bytearray', None),
+    ('recursion_loop', 'Recursion error - cyclic reference detected', None),
+    ('dict_attributes_type', 'Input should be a valid dictionary or instance to extract fields from', None),
+    ('missing', 'Field required', None),
+    ('frozen', 'Field is frozen', None),
+    ('extra_forbidden', 'Extra inputs are not permitted', None),
+    ('invalid_key', 'Keys should be strings', None),
+    ('get_attribute_error', 'Error extracting attribute: foo', {'error': 'foo'}),
+    ('model_class_type', 'Input should be an instance of foo', {'class_name': 'foo'}),
+    ('none_required', 'Input should be None/null', None),
+    ('bool', 'Input should be a valid boolean', None),
+    ('greater_than', 'Input should be greater than 42.1', {'gt': 42.1}),
+    ('greater_than', 'Input should be greater than 42.1', {'gt': '42.1'}),
+    ('greater_than', 'Input should be greater than 2020-01-01', {'gt': '2020-01-01'}),
+    ('greater_than_equal', 'Input should be greater than or equal to 42.1', {'ge': 42.1}),
+    ('less_than', 'Input should be less than 42.1', {'lt': 42.1}),
+    ('less_than_equal', 'Input should be less than or equal to 42.1', {'le': 42.1}),
+    ('finite_number', 'Input should be a finite number', None),
+    (
+        'too_short',
+        'Foobar should have at least 42 items after validation, not 40',
+        {'field_type': 'Foobar', 'min_length': 42, 'actual_length': 40},
+    ),
+    (
+        'too_long',
+        'Foobar should have at most 42 items after validation, not 50',
+        {'field_type': 'Foobar', 'max_length': 42, 'actual_length': 50},
+    ),
+    ('string_type', 'Input should be a valid string', None),
+    ('string_sub_type', 'Input should be a string, not an instance of a subclass of str', None),
+    ('string_unicode', 'Input should be a valid string, unable to parse raw data as a unicode string', None),
+    ('string_pattern_mismatch', "String should match pattern 'foo'", {'pattern': 'foo'}),
+    ('string_too_short', 'String should have at least 42 characters', {'min_length': 42}),
+    ('string_too_long', 'String should have at most 42 characters', {'max_length': 42}),
+    ('dict_type', 'Input should be a valid dictionary', None),
+    ('dict_from_mapping', 'Unable to convert mapping to a dictionary, error: foobar', {'error': 'foobar'}),
+    ('iterable_type', 'Input should be iterable', None),
+    ('iteration_error', 'Error iterating over object, error: foobar', {'error': 'foobar'}),
+    ('list_type', 'Input should be a valid list/array', None),
+    ('tuple_type', 'Input should be a valid tuple', None),
+    ('set_type', 'Input should be a valid set', None),
+    ('bool_type', 'Input should be a valid boolean', None),
+    ('bool_parsing', 'Input should be a valid boolean, unable to interpret input', None),
+    ('int_type', 'Input should be a valid integer', None),
+    ('int_parsing', 'Input should be a valid integer, unable to parse string as an integer', None),
+    ('int_from_float', 'Input should be a valid integer, got a number with a fractional part', None),
+    ('multiple_of', 'Input should be a multiple of 42.1', {'multiple_of': 42.1}),
+    ('greater_than', 'Input should be greater than 42.1', {'gt': 42.1}),
+    ('greater_than_equal', 'Input should be greater than or equal to 42.1', {'ge': 42.1}),
+    ('less_than', 'Input should be less than 42.1', {'lt': 42.1}),
+    ('less_than_equal', 'Input should be less than or equal to 42.1', {'le': 42.1}),
+    ('float_type', 'Input should be a valid number', None),
+    ('float_parsing', 'Input should be a valid number, unable to parse string as an number', None),
+    ('bytes_type', 'Input should be a valid bytes', None),
+    ('bytes_too_short', 'Data should have at least 42 bytes', {'min_length': 42}),
+    ('bytes_too_long', 'Data should have at most 42 bytes', {'max_length': 42}),
+    ('value_error', 'Value error, foobar', {'error': 'foobar'}),
+    ('assertion_error', 'Assertion failed, foobar', {'error': 'foobar'}),
+    ('literal_error', 'Input should be foo', {'expected': 'foo'}),
+    ('literal_error', 'Input should be foo or bar', {'expected': 'foo or bar'}),
+    ('date_type', 'Input should be a valid date', None),
+    ('date_parsing', 'Input should be a valid date in the format YYYY-MM-DD, foobar', {'error': 'foobar'}),
+    ('date_from_datetime_parsing', 'Input should be a valid date or datetime, foobar', {'error': 'foobar'}),
+    ('date_from_datetime_inexact', 'Datetimes provided to dates should have zero time - e.g. be exact dates', None),
+    ('date_past', 'Date should be in the past', None),
+    ('date_future', 'Date should be in the future', None),
+    ('time_type', 'Input should be a valid time', None),
+    ('time_parsing', 'Input should be in a valid time format, foobar', {'error': 'foobar'}),
+    ('datetime_type', 'Input should be a valid datetime', None),
+    ('datetime_parsing', 'Input should be a valid datetime, foobar', {'error': 'foobar'}),
+    ('datetime_object_invalid', 'Invalid datetime object, got foobar', {'error': 'foobar'}),
+    ('datetime_past', 'Datetime should be in the past', None),
+    ('datetime_future', 'Datetime should be in the future', None),
+    ('time_delta_type', 'Input should be a valid timedelta', None),
+    ('time_delta_parsing', 'Input should be a valid timedelta, foobar', {'error': 'foobar'}),
+    ('frozen_set_type', 'Input should be a valid frozenset', None),
+    ('is_instance_of', 'Input should be an instance of Foo', {'class': 'Foo'}),
+    ('is_subclass_of', 'Input should be a subclass of Foo', {'class': 'Foo'}),
+    ('callable_type', 'Input should be callable', None),
+    (
+        'union_tag_invalid',
+        "Input tag 'foo' found using bar does not match any of the expected tags: baz",
+        {'discriminator': 'bar', 'tag': 'foo', 'expected_tags': 'baz'},
+    ),
+    ('union_tag_not_found', 'Unable to extract tag using discriminator foo', {'discriminator': 'foo'}),
+    ('arguments_type', 'Arguments must be a tuple, list or a dictionary', None),
+    ('positional_arguments_type', 'Positional arguments must be a list or tuple', None),
+    ('keyword_arguments_type', 'Keyword arguments must be a dictionary', None),
+    ('unexpected_keyword_argument', 'Unexpected keyword argument', None),
+    ('missing_keyword_argument', 'Missing required keyword argument', None),
+    ('unexpected_positional_argument', 'Unexpected positional argument', None),
+    ('missing_positional_argument', 'Missing required positional argument', None),
+    ('multiple_argument_values', 'Got multiple values for argument', None),
+]
+
+
+@pytest.mark.parametrize('kind, message, context', all_errors)
 def test_error_kind(kind, message, context):
     e = PydanticKindError(kind, context)
     assert e.message() == message
@@ -266,6 +272,12 @@ def test_error_kind(kind, message, context):
     assert e.context == context
 
 
+def test_all_errors_covered():
+    listed_kinds = set(kind for kind, *_ in all_errors)
+    actual_kinds = {e['kind'] for e in list_all_errors()}
+    assert actual_kinds == listed_kinds
+
+
 def test_error_decimal():
     e = PydanticKindError('greater_than', {'gt': Decimal('42.1')})
     assert e.message() == 'Input should be greater than 42.1'
