@@ -4,6 +4,7 @@ from pathlib import Path
 
 import pytest
 
+from pydantic_core import core_schema
 from pydantic_core._pydantic_core import (
     SchemaError,
     SchemaValidator,
@@ -12,7 +13,6 @@ from pydantic_core._pydantic_core import (
     build_profile,
     list_all_errors,
 )
-from pydantic_core.core_schema import ErrorKind
 
 
 @pytest.mark.parametrize('obj', [ValidationError, SchemaValidator, SchemaError])
@@ -174,7 +174,7 @@ def test_all_errors():
         },
     ]
     kinds = [e['kind'] for e in errors]
-    if kinds != list(ErrorKind.__args__):
+    if kinds != list(core_schema.ErrorKind.__args__):
         literal = ''.join(f'\n    {e!r},' for e in kinds)
         print(f'python code:\n\nErrorKind = Literal[{literal}\n]')
         pytest.fail('core_schema.ErrorKind needs to be updated')
