@@ -84,26 +84,53 @@ def test_float_strict(py_and_json: PyAndJson, input_value, expected):
         ({'le': 0}, 0.1, Err('Input should be less than or equal to 0')),
         ({'lt': 0}, 0, Err('Input should be less than 0')),
         ({'lt': 0.123456}, 1, Err('Input should be less than 0.123456')),
-        ({'multiple_of': 0.5}, 0.5, 0.5),
-        ({'multiple_of': 0.5}, 1, 1),
-        ({'multiple_of': 0.5}, 0.6, Err('Input should be a multiple of 0.5')),
     ],
 )
 def test_float_kwargs(py_and_json: PyAndJson, kwargs: Dict[str, Any], input_value, expected):
     v = py_and_json({'type': 'float', **kwargs})
     if isinstance(expected, Err):
-        with pytest.raises(ValidationError, match=re.escape(expected.message)) as exc_info:
+        with pytest.raises(ValidationError, match=re.escape(expected.message)):
             v.validate_test(input_value)
-        errors = exc_info.value.errors()
-        assert len(errors) == 1
-        if 'context' in errors[0]:
-            assert errors[0]['context'] == kwargs
     else:
         output = v.validate_test(input_value)
         assert output == expected
         assert isinstance(output, float)
 
 
+@pytest.mark.parametrize(
+    'multiple_of,input_value,error',
+    [
+        (0.5, 0.5, None),
+        (0.5, 1, None),
+        (0.5, 0.6, Err('Input should be a multiple of 0.5')),
+        (0.5, 0.51, Err('Input should be a multiple of 0.5')),
+        (0.5, 0.501, Err('Input should be a multiple of 0.5')),
+        (0.5, 1_000_000.5, None),
+        (0.5, 1_000_000.49, Err('Input should be a multiple of 0.5')),
+        (0.1, 0, None),
+        (0.1, 0.0, None),
+        (0.1, 0.2, None),
+        (0.1, 0.3, None),
+        (0.1, 0.4, None),
+        (0.1, 0.5, None),
+        (0.1, 0.5001, Err('Input should be a multiple of 0.1')),
+        (0.1, 1, None),
+        (0.1, 1.0, None),
+        (0.1, int(5e10), None),
+    ],
+    ids=repr,
+)
+def test_float_multiple_of(py_and_json: PyAndJson, multiple_of, input_value, error):
+    v = py_and_json({'type': 'float', 'multiple_of': multiple_of})
+    if error:
+        with pytest.raises(ValidationError, match=re.escape(error.message)):
+            v.validate_test(input_value)
+    else:
+        output = v.validate_test(input_value)
+        assert output == input_value
+        assert isinstance(output, float)
+
+
 def test_union_float(py_and_json: PyAndJson):
     v = py_and_json(
         {'type': 'union', 'choices': [{'type': 'float', 'strict': True}, {'type': 'float', 'multiple_of': 7}]}
@@ -187,34 +214,30 @@ def test_float_key(py_and_json: PyAndJson):
     'input_value,allow_inf_nan,expected',
     [
         ('NaN', True, FunctionCheck(math.isnan)),
-        (
-            'NaN',
-            False,
-            Err("Input should be a finite number [kind=float_finite_number, input_value='NaN', input_type=str]"),
-        ),
+        ('NaN', False, Err("Input should be a finite number [kind=finite_number, input_value='NaN', input_type=str]")),
         ('+inf', True, FunctionCheck(lambda x: math.isinf(x) and x > 0)),
         (
             '+inf',
             False,
-            Err("Input should be a finite number [kind=float_finite_number, input_value='+inf', input_type=str]"),
+            Err("Input should be a finite number [kind=finite_number, input_value='+inf', input_type=str]"),
         ),
         ('+infinity', True, FunctionCheck(lambda x: math.isinf(x) and x > 0)),
         (
             '+infinity',
             False,
-            Err("Input should be a finite number [kind=float_finite_number, input_value='+infinity', input_type=str]"),
+            Err("Input should be a finite number [kind=finite_number, input_value='+infinity', input_type=str]"),
         ),
         ('-inf', True, FunctionCheck(lambda x: math.isinf(x) and x < 0)),
         (
             '-inf',
             False,
-            Err("Input should be a finite number [kind=float_finite_number, input_value='-inf', input_type=str]"),
+            Err("Input should be a finite number [kind=finite_number, input_value='-inf', input_type=str]"),
         ),
         ('-infinity', True, FunctionCheck(lambda x: math.isinf(x) and x < 0)),
         (
             '-infinity',
             False,
-            Err("Input should be a finite number [kind=float_finite_number, input_value='-infinity', input_type=str]"),
+            Err("Input should be a finite number [kind=finite_number, input_value='-infinity', input_type=str]"),
         ),
         ('0.7', True, 0.7),
         ('0.7', False, 0.7),
@@ -253,7 +276,7 @@ def test_non_finite_json_values(py_and_json: PyAndJson, input_value, allow_inf_n
         (
             float('nan'),
             False,
-            Err('Input should be a finite number [kind=float_finite_number, input_value=nan, input_type=float]'),
+            Err('Input should be a finite number [kind=finite_number, input_value=nan, input_type=float]'),
         ),
     ],
 )
@@ -273,7 +296,7 @@ def test_non_finite_float_values(strict, input_value, allow_inf_nan, expected):
         (
             float('+inf'),
             False,
-            Err('Input should be a finite number [kind=float_finite_number, input_value=inf, input_type=float]'),
+            Err('Input should be a finite number [kind=finite_number, input_value=inf, input_type=float]'),
         ),
         (
             float('-inf'),
@@ -283,7 +306,7 @@ def test_non_finite_float_values(strict, input_value, allow_inf_nan, expected):
         (
             float('-inf'),
             False,
-            Err('Input should be a finite number [kind=float_finite_number, input_value=-inf, input_type=float]'),
+            Err('Input should be a finite number [kind=finite_number, input_value=-inf, input_type=float]'),
         ),
     ],
 )
